/*  Motti -- a strategy game
    Copyright (C) 1999 Free Software Foundation

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef CURSES_IF_H

#define CURSES_IF_H

#include "comm.h"

enum refresh_amount {NONE, CURSOR, STATUSLINE, REFRESH_MAP, ALL};

extern void curses_game_init (union conn_info *);
extern void kill_curses (void);
extern void c_draw_loc (const int);
extern void c_read_key (union conn_info *);
extern void do_refresh (const enum refresh_amount);

#endif /* curses_if.h */
